<?php

/*
 * Plugin Name: 1Pay Payments | WooCommerce
 * Plugin URI: http://1pay.in
 * Description: Extends WooCommerce with 1Pay services
 * Version: 1.0.3
 * Author: Soumil Gada, Manisha Gupta
 * Author URI: http://mobilewaretech.com/soumilgada
 * Copyright: @2018 Mobilewaretech. All rights reserved
 */

$bd = ABSPATH . 'wp-content/plugins' . dirname(plugin_basename(__FILE__));
add_action('plugins_loaded', 'wc_1pay_init', 0);

function wc_1pay_init() {
    if (!class_exists('WC_Payment_Gateway'))
        return;

    /*
     * Localisation
     */
    load_plugin_textdomain('wc-1pay', false, dirname(plugin_basename(__FILE__)) . '/languages');

    /*
     * PG Class Declaration
     */

    class WC_1Pay extends WC_Payment_Gateway {

        private $ekey;

        public function __construct() {
            global $wpdb;
            $this->ekey = '3a5c1c4e81d7eb133a5c1c4e81d7eb13';
            $this->id = '1pay';
            $this->method_title = __('1Pay', '1pay');
            $this->icon = WP_PLUGIN_URL . "/" . plugin_basename(dirname(__FILE__)) . "/logo.png";
            $this->has_fields = false;
            $this->init_form_fields();
            $this->init_settings();
            $this->title = '1Pay';
            $this->description = $this->settings['Description'];
            $this->transaction_mode = $this->settings['transaction_mode'];
            $this->redirect_page_id = $this->settings['redirect_page_id'];
            $this->merchantid = $this->settings['mid'];
            $this->apiKey = $this->settings['apikey'];
            $this->encryptionkey = $this->settings['enc_key'];
            $this->msg['message'] = "";
            $this->msg['class'] = "";

            add_action('init', array(&$this, 'check_response'));

            add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'check_response'));

            add_action('valid-request', array(&$this, 'SUCCESS'));

            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0.', '>=')) {
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
            } else {
                add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            }

            add_action('woocommerce_receipt_1pay', array(&$this, 'receipt_page'));
            add_action('woocommerce_thankyou_1pay', array(&$this, 'thankyou_page'));

            $this->logger = wc_get_logger();
        }

        function init_form_fields() {
            $this->form_fields = array(
                "enabled" => array(
                    'title' => __('Enable/Disable', '1pay'),
                    'type' => 'checkbox',
                    'label' => __('Enable 1Pay', '1pay'),
                    'default' => 'no'
                ),
                'description' => array(
                    'title' => __('Description:', '1pay'),
                    'type' => 'textarea',
                    'description' => __('This controls the description which the user sees during checkout.', '1pay'),
                    'default' => __('Pay securely by Credit or Debit card or net banking through 1pay.', '1pay')),
                'transaction_mode' => array(
                    'title' => __('Transaction Mode', '1pay'),
                    'type' => 'select',
                    'options' => array("0" => "Select", "test" => "Test", "live" => "Live"),
                    'description' => __('Transaction Mode.', '1pay')
                ),
                'mid' => array(
                    'title' => __('Merchant ID', '1pay'),
                    'type' => 'text',
                    'description' => __('1pay merchant ID.', '1pay')
                ),
                'apikey' => array(
                    'title' => __('API Key', '1pay'),
                    'type' => 'text',
                    'description' => __('1pay API Key.', '1pay')
                ),
                'redirect_page_id' => array(
                    'title' => __('Return Page'),
                    'type' => 'select',
                    'options' => $this->get_pages('Select Page'),
                    'description' => "URL of success page"
                )
            );
        }

        /*
         * Admin Panel Options
         * - Options for bits like 'title' and availability on basis
         */

        function admin_options() {
            echo "<h3>" . __('1Pay payment', '1pay') . '</h3>';
            echo "<p>" . __("1Pay India's leading Payment Provider", "1pay") . '</p>';

            echo '<table class="form-table">';
            $this->generate_settings_html();
            echo "</table>";
        }

        /*
         * There are no payment fields, but we would like to show the description if set.
         */

        function payment_fields() {
            if ($this->description)
                echo wpautop(wptexturize($this->description));
        }

        /*
         * Receipt Page
         */

        function receipt_page($order) {
            echo "<p>" . __('Thank you for your order, plpease click the button below to pay.', '1pay') . '</p>';
            echo $this->generate_form($order);
        }

        /*
         * Process payment and return result
         */

        function process_payment($order_id) {
            $order = new WC_Order($order_id);
            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                return array(
                    'result' => 'success',
                    'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, $order->get_checkout_payment_url(true)))
                );
            } else {
                return array(
                    'result' => 'success',
                    'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(get_option('woocommerce_pay_page_id'))))
                );
            }
        }

        function check_response() {

            // @TODO Response as per 1Pay

            global $woocommerce;

            $respData = $_POST['respData'];
            $respDecrypt = $this->get_decrypt($respData);
            $verificationResult = json_decode($respDecrypt);

            $txnid = $verificationResult->txn_id;
            $merchantid = $verificationResult->merchant_id;

            $url = 'https://pay.1paypg.in/payone/verifyTxn?';
            if ($this->transaction_mode == 'test') {
                $url = 'http://139.59.1.254:8080/onepayVAS/verifyTxn?';
                $collaborator = '1Pay';
            }


            $myvars = "merchantId=" . $merchantid . "&txnId=" . $txnid . "";

            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $myvars);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $verify = curl_exec($ch);


            $verificationResult = json_decode($verify);

            $parsedtxnid = $verificationResult->txn_id;
            $parsedmerchantid = $verificationResult->merchant_id;
            $parsedtxnamount = $verificationResult->txn_amount;
            $parsedtxnstatus = $verificationResult->trans_status;
            $responseMessage = $verificationResult->resp_message;

            if (($responseMessage === 'No Error.') && ($parsedtxnstatus !== 'NA')) {

                $vmessage = "Verified Transaction";
                if ($parsedtxnstatus == 'Ok') {

                    $orderid = explode('-', $parsedtxnid);
                    $orderid = (int) $orderid[0];    //get rid of time part
                    $order = new WC_Order($orderid);

                    $this->msg['class'] = 'error';
                    $this->msg['message'] = "Thank you for shopping with us. However, the transaction has been declined.";

                   
                    $this->msg['message'] = "Thank you for shopping with us. Your account has been charged and your transaction is successful with following order details: 
								
							<br> 
								Order Id: $orderid <br/>
								Amount: $parsedtxnamount 
								<br />
								
									
						We will be shipping your order to you soon.";

                    $this->msg['class'] = 'success';

                    if ($order->status == 'processing' || $order->status == 'completed') {
                        //do nothing
                    } else {
                        //complete the order
                        $order->payment_complete();
                        $order->add_order_note('1Pay has processed the payment. Ref Number: ' . $orderid);
                        $order->add_order_note($this->msg['message']);
                        $order->add_order_note("Paid by 1Pay");
                        $woocommerce->cart->empty_cart();
                    }
                } else if ($parsedtxnstatus == 'To') {
                    $this->msg['class'] = 'error';
                    $this->msg['message'] = "Sorry!!Your Transaction is Timed Out";
                } else {
                    $this->msg['class'] = 'error';
                    $this->msg['message'] = "Transaction Failed";
                }
            } else {
                $vmessage = "Transaction Failed";


                $this->msg['class'] = 'error';
                $this->msg['message'] = $vmessage;
            }



            //manage msessages
            if (function_exists('wc_add_notice')) {
                wc_add_notice($this->msg['message'], $this->msg['class']);
            } else {
                if ($this->msg['class'] == 'success') {
                    $woocommerce->add_message($this->msg['message']);
                } else {
                    $woocommerce->add_error($this->msg['message']);
                }
                $woocommerce->set_messages();
            }

            $redirect_url = ($this->redirect_page_id == "" || $this->redirect_page_id == 0) ? get_site_url() . "/" : get_permalink($this->redirect_page_id);
            wp_redirect($redirect_url);
            exit;
        }

        public function generate_form($order_id) {
            global $woocommerce;
            $order = new WC_Order($order_id);
            $redirect_url = ($this->redirect_page_id == "" || $this->redirect_page_id == 0) ? get_site_url() . "/" : get_permalink($this->redirect_page_id);
            $redirect_url = $redirect_url . 'wc-api/' . get_class($this);
            $order_id = $order_id . '_' . date("ymds");
            $action = 'https://pay.1paypg.in/';
            if ($this->transaction_mode == 'test') {
                $action = 'http://139.59.1.254:8080/';
                $collaborator = '1Pay';
            }
            $amount = $order->order_total;
            $currency = get_woocommerce_currency();
            $dateTime = date('Y-m-d h:i:s');
            $txnid = $order_id; //$this->merchantid.'-'.uniqid();
            $checksum = $this->get_checksum(array('date' => $dateTime, 'txnId' => $txnid, 'amount' => $amount, 'custMobile' => $order->billing_phone, 'custEmail' => $order->billing_email, 'action' => $action));
            $order_data = array(
                'dateTime' => $dateTime,
                'amount' => $amount,
                'isMultiSettlement' => '0',
                'custMobile' => $order->billing_phone,
                'apiKey' => $this->apiKey,
                'productId' => 'DEFAULT',
                'instrumentId' => 'NA',
                'cardType' => 'NA',
                'txnType' => 'DIRECT',
                'udf3' => 'NA',
                'udf1' => 'NA',
                'udf2' => 'NA',
                'merchantId' => $this->merchantid,
                'custMail' => $order->billing_email,
                'returnURL' => $redirect_url,
                'channelId' => '0',
                'txnId' => $txnid,
                'checksum' => $checksum,
                'bankId' => 'NA',
                'cardDetails' => 'NA',
                'udf4' => 'NA',
                'udf5' => 'NA',
            );

            $jsondata = json_encode($order_data);

            $enc = $this->get_encrypt($jsondata);

            $form = '<html><body><form name="1pay_form" id="1pay_form" method="post" action="' . $action . 'onepayVAS/payprocessor">
                        <input type="text" name="merchantId" value="' . $this->merchantid . '">
                        <input type="text" name="reqData" value="' . $enc . '">
                    </form> 
                    <script type="text/javascript">document.getElementById("1pay_form").submit();</script>
                    </body></html>';
            return $form;
        }

        function get_checksum($data) {
            $datetime = $data['date'];
            $merchantId = $this->merchantid;
            $apiKey = $this->apiKey;
            $txnId = $data['txnId'];
            $amount = $data['amount'];
            $mobileno = $data['custMobile'];
            $email = $data['custEmail'];
            $txntype = 'DIRECT';
            $checkSumUrl = $data['action'] . 'payone/checksumgenerator';
            $myvars = "merchantId=" . $merchantId . "&txnId=" . $txnId . "&dateTime=" . $datetime . "&amount=" . $amount . "&apiKey=" . $apiKey . "&custMobile=" . $mobileno . "&custMail=" . $email . "&channelId=0&txnType=DIRECT";
            $ch = curl_init($checkSumUrl);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $myvars);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $checksum = curl_exec($ch);
            $value = json_decode($checksum);
            return $value->checksum;
        }

        function get_pages($title = false, $indent = true) {
            $wp_pages = get_pages('sort_column=menu_order');
            $page_list = array();
            if ($title)
                $page_list[] = $title;
            foreach ($wp_pages as $page) {
                $prefix = '';
                // show indented child pages?
                if ($indent) {
                    $has_parent = $page->post_parent;
                    while ($has_parent) {
                        $prefix .= ' - ';
                        $next_page = get_page($has_parent);
                        $has_parent = $next_page->post_parent;
                    }
                }
                // add to page list array array
                $page_list[$page->ID] = $prefix . $page->post_title;
            }
            return $page_list;
        }

        public function get_encrypt($input) {
            //for php 5.x.x to 7.0.x
            $key = $this->ekey;
            $size = mcrypt_get_block_size(MCRYPT_RIJNDAEL_128, MCRYPT_MODE_ECB);

            $input = $this->get_pkcs5_pad($input, $size);

            $td = mcrypt_module_open(MCRYPT_RIJNDAEL_128, '', MCRYPT_MODE_ECB, '');

            $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);

            mcrypt_generic_init($td, $key, $iv);
            $data = mcrypt_generic($td, $input);

            mcrypt_generic_deinit($td);
            mcrypt_module_close($td);
            $data = base64_encode($data);

            //die;
            return $data;
        }

        protected function get_pkcs5_pad($text, $blocksize) {
            $pad = $blocksize - (strlen($text) % $blocksize);
            return $text . str_repeat(chr($pad), $pad);
        }

        public function get_decrypt($input) {
            $key = $this->ekey;
            $decrypted = mcrypt_decrypt(
                    MCRYPT_RIJNDAEL_128, $key, base64_decode($input), MCRYPT_MODE_ECB
            );
            $dec_s = strlen($decrypted);
            $padding = ord($decrypted[$dec_s - 1]);
            $decrypted = substr($decrypted, 0, -$padding);


            return $decrypted;
        }

    }

    function woocommerce_add_gateway($methods) {
        $methods[] = 'WC_1Pay';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'woocommerce_add_gateway');
}

?>